#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
pyzufall.generator
------------------

Stellt diverse Generator-Funktionen zur Verfügung.
"""

import re
import random
from datetime import timedelta, date

from .helfer import lese, chance, str_add, uml

# Regex Pattern für Doctests
from pyzufall.helfer import re_wort, re_worte, re_liste, re_datum, re_email

# Benutze /dev/urandom oder Windows CryptGenRandom für bessere Entropy
r = random.SystemRandom()

# Namen einlesen
vornamen_m = lese('vornamen_m.txt')
vornamen_w = lese('vornamen_w.txt')
nachnamen = lese('nachnamen.txt')

# Objekte einlesen
pflanzen = lese('pflanzen.txt')
baeume = lese('baeume.txt')
tiere = lese('tiere.txt')
gegenstaende = lese('gegenstaende.txt')
koerperteile = lese('koerperteile.txt')
nahrung = lese('nahrung.txt')
geschmack = lese('geschmack.txt')
berufe = lese('berufe.txt')
musik = lese('musikgenre.txt')
stadte = lese('stadt_bundesland.txt')
interessen = lese('interessen.txt')
domains = lese('email_domains.txt')

# Wortarten einlesen

# Verben
nullwertige_verben = lese('nullwertige_verben.txt')
intransitive_verben = lese('intransitive_verben.txt')
intransitive_verben_2 = lese('intransitive_verben_2.txt')
transitive_verben = lese('transitive_verben.txt')
transitive_verben_2 = lese('transitive_verben_2.txt')
ditransitive_verben = lese('ditransitive_verben.txt')

# Adjektive
adjektive = lese('adjektiv.txt')

# Sazteile oder Sätze
ortsangabe = lese('ort.txt')
ns = lese('nebensatz.txt')
sprichwoerter = lese('sprichwoerter.txt')


def geschlecht():
	"""
	Gibt ein zufälliges Geschlecht zurück.

	1 = männlich
	0 = weiblich

	2011 gibt es laut Statistik 51,18% weibliche Personen in Deutschland:
	https://www.destatis.de/DE/ZahlenFakten/GesellschaftStaat/Bevoelkerung/Bevoelkerungsstand/Tabellen/Zensus_Geschlecht_Staatsangehoerigkeit.html

	:rtype: int

	.. only:: doctest

		>>> s = geschlecht()

		>>> assert s is 0 or 1
	"""
	if r.randint(1, 100) <= 51:
		return 0
	else:
		return 1


def vorname_m():
	"""
	Gibt einen männlichen Vornamen zurück.

	.. only:: doctest

		>>> s = vorname_m()

		testet ob ergebnis in liste ist
		>>> assert s in vornamen_m
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(vornamen_m)


def vorname_w():
	"""
	Gibt einen weiblichen Vornamen zurück.

	.. only:: doctest

		>>> s = vorname_w()

		testet ob ergebnis in liste ist
		>>> assert s in vornamen_w
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(vornamen_w)


def vorname():
	"""
	Gibt einen Vornamen zurück.
	"""
	return r.choice([vorname_m, vorname_w])()


def nachname():
	"""
	Gibt einen Nachnamen zurück.

	.. only:: doctest

		>>> s = nachname()

		testet ob ergebnis in liste ist
		>>> assert s in nachnamen
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(nachnamen)


def nickname(vorname='', nachname=''):
	"""
	Generiert einen Nickname, Angabe von Vor- und Nachname optional.

	Beispiel: dicker_falke, beate_brutal85, stinkender_panda24

	.. versionadded:: 0.13
	"""
	if vorname and nachname and r.randint(1,100) <= 75:
		x =  r.randint(1, 5)
		if x == 1:
			s = vorname[0] + nachname[:2]
		elif x == 2:
			s = vorname[:r.randint(2, len(vorname))] + nachname[:r.randint(2, len(nachname))]
		elif x == 3:
			s = vorname[:r.randint(3, len(vorname))] + wort(r.randint(3, 5))
		elif x == 4:
			s = vorname
			y = objekt().split(' ', 1)[1]
			i = 0
			while ( y[0].lower() != s[0].lower() and i < 250 ):
				if i % 2 == 0:
					y = objekt().split(' ', 1)[1]
				else:
					y = adjektiv()
				i += 1
			s = s + '_' + y
			s = s.replace('-', '_')
		elif x == 5:
			s = vorname + verbi()
			s = s.replace(' ', '')
	else:
		x =  r.randint(1, 2)
		if x == 1:
			s = objekt().replace(' ', '_')
			s = s.replace('-', '_')
		elif x == 2:
			s = objekt()
			if s.split(' ', 1)[0] == 'der':
				s = str_add(adjektiv(), 'er') + '_' + s.split(' ', 1)[1]
			else:
				s = str_add(adjektiv(), 'e') + '_' + s.split(' ', 1)[1]
			s = s.replace('-', '_')

	return s.lower() + chance(20, str(r.randint(1,99)))


def homepage(vorname, nachname, nick=''):
	"""
	Gibt die Domain einer persönlichen Homepage zurück.

	Beispiel: lilim.eu, heruntergekommene-kastanie.net, damian-schuett.org

	.. versionadded:: 0.13
	"""
	x = r.randint(1,8)
	if x == 1 or x == 2:
		s = vorname + '-' + nachname
	elif x == 3:
		s = vorname + '.' + nachname
	elif x == 4:
		s = vorname[0] + nachname
	elif x == 5:
		s = vorname + nachname[0]
	elif x == 6:
		s = nachname
	elif nick:
		s = nick
	else:
		s = nickname(vorname, nachname)
	s = s.replace('_', '-')

	return uml(s.lower()) + '.' + r.choice(['de', 'net', 'org', 'eu', 'com', 'me'])


def url(domain):
	"""
	>>> url('davidak.de')
	'http://davidak.de/'

	.. versionadded:: 0.13

	.. only:: doctest

		>>> s = url('säge.de')

		>>> print(s)
		http://säge.de/
	"""
	return 'http://{}/'.format(domain)


def email(vorname, nachname, nick='', domain=''):
	"""
	Generiert eine E-Mail-Adresse.

	Beispiel: ismail@ismail-christ.eu, emelieeiru63@lebanese.cc, nic@copacabana.com

	.. only:: doctest

		>>> s = email('Bernd', 'Lieferts')

		>>> assert re.match(re_email, s)

	.. versionadded:: 0.13
	"""
	if nick == '':
		nick = nickname(vorname, nachname)

	if domain and r.randint(1, 100) < 60:
		s = r.choice([vorname, nick]) + '@' + domain
	else:
		s = r.choice([vorname + '.' + nachname, vorname, nachname, nick]) + '@' + r.choice(domains)

	return uml(s.lower())


def verbn():
	"""
	Gibt ein nullwertiges Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbn()

		testst ob ergebnis in liste ist
		>>> assert s in nullwertige_verben
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(nullwertige_verben)


def verbi():
	"""
	Gibt ein intransitives Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbi()

		testet ob ergebnis in liste ist
		>>> assert s in intransitive_verben
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(intransitive_verben)


def verbi2():
	"""
	Gibt ein intransitives, getrenntes Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbi2()

		testet ob ergebnis in liste ist
		>>> assert s in intransitive_verben_2
		
		>>> assert re.match(re_liste, s)
	"""
	return r.choice(intransitive_verben_2)


def verbt():
	"""
	Gibt ein transitives Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbt()

		testet ob ergebnis in liste ist
		>>> assert s in transitive_verben
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(transitive_verben)


def verbt2():
	"""
	Gibt ein intransitives, getrenntes Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbt2()

		testet ob ergebnis in liste ist
		>>> assert s in transitive_verben_2
		
		>>> assert re.match(re_liste, s)
	"""
	return r.choice(transitive_verben_2)


def verbd():
	"""
	Gibt ein ditransitives Verb zurück.

	`Beschreibung auf Wikipedia <http://de.wikipedia.org/wiki/Transitivität_(Grammatik)#Festlegung_der_Transitivit.C3.A4t_eines_Verbs/>`_

	.. only:: doctest

		>>> s = verbd()

		testet ob ergebnis in liste ist
		>>> assert s in ditransitive_verben
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(ditransitive_verben)


def adjektiv():
	"""
	Gibt ein Adjektiv zurück.

	.. only:: doctest

		>>> s = adjektiv()

		testet ob ergebnis in liste ist
		>>> assert s in adjektive
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(adjektive)


def gegenstand():
	"""
	Gibt einen Gegenstand zurück.

	.. only:: doctest

		>>> s = gegenstand()

		testet ob ergebnis in liste ist
		>>> assert s in gegenstaende
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(gegenstaende)


def koerperteil():
	"""
	Gibt ein Körperteil zurück.

	.. only:: doctest

		>>> s = koerperteil()

		testet ob ergebnis in liste ist
		>>> assert s in koerperteile
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(koerperteile)


def interesse():
	"""
	Gibt ein zufälliges Interesse bzw Hobby zurück.

	.. deprecated:: 0.11
		Wird durch :func:`pyzufall.generator.interessen_liste` ersetzt.

	.. only:: doctest

		>>> s = interesse()

		testet ob ergebnis in liste ist
		>>> assert s in interessen
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(interessen)


def tier():
	"""
	Gibt ein Tier zurück.

	.. only:: doctest

		>>> s = tier()

		testet ob ergebnis in liste ist
		>>> assert s in tiere
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(tiere)


def baum():
	"""
	Gibt einen Baum zurück.

	.. only:: doctest

		>>> s = baum()

		testet ob ergebnis in liste ist
		>>> assert s in baeume
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(baeume)


def pflanze():
	"""
	Gibt eine Pflanze zurück.

	.. only:: doctest

		>>> s = pflanze()

		testet ob ergebnis in liste ist
		>>> assert s in pflanzen
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(pflanzen)


beziehung_m = ['Vater', 'Bruder', 'Mann', 'Sohn', 'Onkel', 'Opa', 'Cousin', 'Enkel', 'Chef', 'Freund', 'Partner', 'Kollege', 'Mitarbeiter', 'Mitbewohner', 'Vermieter', 'Lehrer']
beziehung_w = ['Mutter', 'Schwester', 'Frau', 'Tochter', 'Tante', 'Oma', 'Cousine', 'Enkelin', 'Cheffin', 'Freundin', 'Partnerin', 'Kollegin', 'Mitarbeiterin', 'Mitbewohnerin', 'Vermieterin', 'Lehrerin']
possessivpronomen_m = ['mein', 'dein', 'sein', 'ihr']
personen = ['er', 'sie', 'es', 'jemand', 'niemand']
spezial_m = ['ein Held', 'ein Penner', 'ein Verkäufer', 'ein Zuhälter', 'ein Lehrer', 'ein Polizist', 'ein Beamter', 'ein Arzt', 'Hitler', 'ein Bernd', 'ein Schwuler', 'ein Behinderter', 'der Affenmensch', 'der Satanist', 'der Alkoholiker', 'ein normaler Mensch', 'ein Pirat', 'ein Hartz-IV-Empfänger', 'ein Müllmann']
spezial_w = ['eine Heldin', 'eine Pennerin', 'eine Verkäuferin', 'eine Zuhälterin', 'eine Prostituierte', 'eine Nutte', 'eine Hure', 'eine Schlampe', 'eine Lehrerin', 'eine Polizistin', 'eine Beamtin', 'eine Ärztin', 'eine Behinderte', 'eine Sekretärin', 'die Transe', 'das Mannsweib', 'das Penismädchen', 'die Lesbe', 'die Kampflesbe', 'die Satanistin', 'die Alkoholikerin', 'die Piratin', 'die Hartz-IV-Empfängerin']
def person_m():
	"""
	Gibt eine männliche Person zurück.

	.. only:: doctest

		>>> s = person_m()

		>>> assert re.match(re_worte, s)
	"""
	z = r.randint(1,10)
	if z == 1:
		s = chance(25, str_add(adjektiv(), 'er '))
		s = r.choice([vorname_m(), vorname_w()]) + 's ' + s + r.choice(beziehung_m)
	elif z == 2:
		s = chance(25, str_add(adjektiv(), 'er '))
		s = r.choice(possessivpronomen_m) + ' ' + s + r.choice(beziehung_m)
	elif z == 3:
		s =  chance(50, str_add(adjektiv(), 'e '))
		s = 'der ' + s + r.choice(beziehung_m)
	elif z == 4:
		s = str_add(adjektiv(), 'e ')
		s = 'der ' + s + r.choice(vornamen_m)
	elif z == 5:
		s = str_add(adjektiv(), 'e')
		s = 'der ' + s.capitalize()
	elif z == 6:
		s = r.choice(['der ', 'ein ']) + beruf_m()
	elif z == 7:
		s = r.choice(spezial_m)
	elif z == 8:
		s = 'er'
	else:
		s = vorname_m()
	return s


def person_w():
	"""
	Gibt eine weibliche Person zurück.

	.. only:: doctest

		>>> s = person_w()

		>>> assert re.match(re_worte, s)
	"""
	z = r.randint(1,10)
	if z == 1:
		s = chance(25, str_add(adjektiv(), 'e '))
		s = r.choice([vorname_m(), vorname_w()]) + 's ' + s + r.choice(beziehung_w)
	elif z == 2:
		s = chance(25, str_add(adjektiv(), 'e '))
		s = r.choice(possessivpronomen_m) + 'e ' + s + r.choice(beziehung_w)
	elif z == 3:
		s =  chance(50, str_add(adjektiv(), 'e '))
		s = 'die ' + s + r.choice(beziehung_w)
	elif z == 4:
		s = str_add(adjektiv(), 'e ')
		s = 'die ' + s + vorname_w()
	elif z == 5:
		s = str_add(adjektiv(), 'e')
		s = 'die ' + s.capitalize()
	elif z == 6:
		s = r.choice(['die ', 'eine ']) + beruf_w()
	elif z == 7:
		s = r.choice(spezial_w)
	elif z == 8:
		s = 'sie'
	else:
		s = vorname_w()
	return s


def person():
	"""
	Gibt eine zufällige Person zurück.
	"""
	return r.choice([person_m, person_w])()


vokal = ['a', 'e', 'i', 'o', 'u', 'ei', 'au']
konsonant = ['b', 'c', 'd', 'f', 'g', 'h', 'k', 'l', 'm', 'n', 'p', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z', 'sch']
konsonant2 = ['h', 'k', 'l', 'm', 'n', 's', 't']
def wort(laenge=r.randint(3,12)):
	"""
	Gibt ein Fantasiewort zurück.

	:param laenge: int

	.. only:: doctest

		>>> s = wort()

		>>> assert re.match(re_wort, s)
	"""
	anfaengt = r.randint(1,3)
	s = ""
	
	if anfaengt == 2: # Anfang wird ein Vokal
		s = r.choice(vokal)
		while len(s) < laenge:
			s += r.choice(konsonant)
			if len(s) < laenge and r.randint(0,1):
				s += r.choice(konsonant2)
			if len(s) >= laenge:
				break
			s += r.choice(vokal)

	else: # Anfang wird ein Konsonant
		s = r.choice(konsonant)
		while len(s) < laenge:
			s += r.choice(vokal)
			if len(s) >= laenge:
				break
			s += r.choice(konsonant)
			if len(s) < laenge and r.randint(0,1):
				s += r.choice(konsonant2)
	s = s.capitalize()
	return s


def zahl():
	"""
	Gibt eine Zahl zwischen 0 und 100 zurück.

	:rtype: string

	.. only:: doctest

		>>> s = zahl()

		>>> assert s.isdigit()
	"""
	s = r.randint(0,100)
	return str(s)


farben = ['Rot', 'Gelb', 'Blau', 'Violett', 'Türkis', 'Orange', 'Grün', 'Magenta', 'Braun', 'Grau', 'Weiß', 'Schwarz']
def farbe():
	"""
	Gibt eine Farbe zurück.

	.. only:: doctest

		>>> s = farbe()

		testet ob ergebnis in liste ist
		>>> assert s in farben
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(farben)


colors = ['red', 'yellow', 'blue', 'violet', 'orange', 'green', 'magenta', 'brown', 'gray', 'white', 'black']
def color():
	"""
	Gibt eine Farbe auf englisch zurück.

	.. only:: doctest

		>>> s = color()

		testet ob ergebnis in liste ist
		>>> assert s in colors
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_wort, s)
	"""
	return r.choice(colors)


def zeitpunkt(start, ende):
	"""
	Gibt einen zufälligen Zeitpunkt (datetime.date) zwischen zwei Zeitpunkten zurück.
	Es handelt sich dabei um ein gültiges Datum.

	.. versionadded:: 0.13

	:param: start, ende: datetime
	:rtype: datetime

	.. only:: doctest

		>>> _heute = date.today()

		>>> _datum = date(2000, 3, 27)

		>>> s = zeitpunkt(_datum, _heute)

		>>> assert re.match(re_datum, s.strftime("%d.%m.%Y"))
	"""
	for arg in [start, ende]:
		if type(arg) is not date:
			raise TypeError("Es muss ein datetime.date Objekt übergeben werden, nicht " + str(type(arg)))

	delta = ende - start
	int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
	rand_second = random.randrange(int_delta)
	return start + timedelta(seconds=rand_second)


def geburtsdatum():
	"""
	Gibt ein gültiges Datum zwischen <vor 110 Jahren> und <heute> zurück.

	:rtype: string

	.. only:: doctest

		>>> s = geburtsdatum()

		>>> assert re.match(re_datum, s)
	"""
	_vor_110_jahren = date.today() - timedelta(days=(110 * 365.24))
	_datum = zeitpunkt(_vor_110_jahren, date.today())
	return _datum.strftime("%d.%m.%Y")


def datum():
	"""
	Gibt ein gültiges Datum zwischen <vor 50 Jahren> und <heute> zurück.

	:rtype: string

	.. only:: doctest

		>>> s = datum()

		>>> assert re.match(re_datum, s)
	"""
	_vor_50_jahren = date.today() - timedelta(days=(50 * 365.24))
	_datum = zeitpunkt(_vor_50_jahren, date.today())
	return _datum.strftime("%d.%m.%Y")


def sprichwort():
	"""
	Gibt ein Sprichwort zurück.

	.. only:: doctest

		>>> s = sprichwort()

		testet ob ergebnis in liste ist
		>>> assert s in sprichwoerter
	"""
	return r.choice(sprichwoerter)


def beruf_m():
	"""
	Gibt eine männliche Berufsbezeichnung zurück.

	.. only:: doctest

		>>> s = beruf_m()

		testet ob ergebnis in liste ist
		>>> assert s in berufe
		
		testet ob ergebnis aus buchstaben besteht
		>>> assert re.match(re_worte, s)
	"""
	return r.choice(berufe)


def beruf_w():
	"""
	Gibt eine weibliche Berufsbezeichnung zurück.

	.. only:: doctest

		>>> s = beruf_w()
		
		>>> assert re.match(re_worte, s)
	"""
	s = beruf_m() + 'in'
	s = re.sub('mannin$', 'frau', s) # Restaurantfachmannin => Restaurantfachfrau
	s = re.sub('kraftin$', 'kraft', s) # Edv-Fachkraftin => Edv-Fachkraft
	s = re.sub('angestellterin$', 'angestellte', s)
	s = re.sub('Fotomodellin$', 'Fotomodell', s)
	s = re.sub('Technischer Zeichnerin$', 'Technische Zeichnerin', s)
	s = re.sub('Arztin$', 'Ärztin', s)
	s = re.sub('arztin$', 'ärztin', s)
	s = re.sub('beamterin$', 'beamtin', s) # Polizeibeamterin => Polizeibeamte
	s = re.sub('stewardin$', 'stewardess', s)
	s = re.sub('Etagenaufsichtin$', 'Etagenaufsicht', s)
	s = re.sub('ein$', 'in', s) # Mikrobiologein => Mikrobiologin
	return s


def objekt():
	"""
	Gibt ein Objekt zurück.
	"""
	return r.choice([gegenstand, tier, pflanze, baum])()


def objekt_m(s):
	"""
	Bringt ein Objekt in Berzug zu einer männlichen Person.

	Beispiel:
	'der Bär' wird zu 'den Bären' oder 'seinen Bären'
	"""
	if 'der ' in s:
		s = re.sub('e$', 'en', s) # Löwen
	s = re.sub('Bär$', 'Bären', s)
	s = re.sub('der ', 'den ', s)
	
	if r.randint(0,2):
		s = re.sub('den ', r.choice(['einen ', 'seinen ', 'deinen ']), s)
		s = re.sub('die ', r.choice(['eine ', 'seine ', 'deine ']), s)
		s = re.sub('das ', r.choice(['ein ', 'sein ', 'dein ']), s)
	
	return s


def objekt_w(s):
	"""
	Bringt ein Objekt in Berzug zu einer weiblichen Person.

	Beispiel:
	'der Bär' wird zu 'den Bären' oder 'ihren Bären'
	"""
	if 'der ' in s:
		s = re.sub('e$', 'en', s) # Löwen
	s = re.sub('Bär$', 'Bären', s)
	s = re.sub('der ', 'den ', s)
	
	if r.randint(0,2):
		s = re.sub('den ', r.choice(['einen ', 'ihren ', 'deinen ']), s)
		s = re.sub('die ', r.choice(['eine ', 'ihre ', 'deine ']), s)
		s = re.sub('das ', r.choice(['ein ', 'ihr ', 'dein ']), s)
	
	return s


def person_objekt_m():
	"""
	Gibt eine Person als Objekt in Bezug auf eine männliche Person zurück.

	Beispiel: seine Mitarbeiterin

	.. only:: doctest

		>>> s = person_objekt_m()
		
		>>> assert re.match(re_worte, s)
	"""
	y = r.randint(1,4)
	if y == 1:
		s = vorname_m()
	if y == 2:
		s = vorname_w()
	if y == 3:
		s =  chance(25, str_add(adjektiv(), 'en '))
		s = r.choice(['seinen ', 'deinen ']) + s + r.choice(beziehung_m)
		s = re.sub('Kollege', 'Kollegen', s)
	if y == 4:
		s =  chance(25, str_add(adjektiv(), 'e '))
		s = r.choice(['seine ', 'deine ']) + s + r.choice(beziehung_w)
		
	return s


def person_objekt_w():
	"""
	Gibt eine Person als Objekt in Bezug auf eine weibliche Person zurück.

	Beispiel: ihre Mutter

	.. only:: doctest

		>>> s = person_objekt_w()
		
		>>> assert re.match(re_worte, s)
	"""
	y = r.randint(1,4)
	if y == 1:
		s = vorname_m()
	if y == 2:
		s = vorname_w()
	if y == 3:
		s =  chance(25, str_add(adjektiv(), 'en '))
		s = r.choice(['ihren ', 'deinen ']) + s + r.choice(beziehung_m)
		s = re.sub('Kollege', 'Kollegen', s)
	if y == 4:
		s =  chance(25, str_add(adjektiv(), 'e '))
		s = r.choice(['ihre ', 'deine ']) + s + r.choice(beziehung_w)

	return s


def ort():
	"""
	Gibt eine Ortsangabe zurück.

	Beispiel: 'im Flur'

	.. todo::

		aufteilen in generator und zufällige aus liste

	.. only:: doctest

		>>> s = ort()
		
		>>> assert re.match(r'^[A-Za-z0-9äÄöÖüÜß -]+$', s)
	"""
	tausende = ['500', '1000', '2000', '5000', '10000', '100000']
	if r.randint(0,3):
		zuschauer = r.choice(tausende)
	else:
		zuschauer = str(r.randint(1,10000))

	s = r.choice(ortsangabe)
	s = re.sub('auf Gleis XX$', 'auf Gleis ' + str(r.randint(1,14)), s)
	s = re.sub('XXZusch', zuschauer, s)
	s = re.sub('XXName', r.choice([vorname_m(), vorname_w()]) + 's', s)
	return s


voressen = ['Brat', 'Rühr', 'Reibe', 'Brech', 'Back', 'Ofen', 'Hack', 'Mager', 'Frisch', 'Bio', 'Gammel', 'Soja', 'Tofu', 'Milch']
def essen():
	"""
	Gibt ein Essen zurück.

	.. only:: doctest

		>>> s = essen()
		
		>>> assert re.match(re_wort, s)
	"""
	if r.randint(0,2): # zusammengesetzt aus zwei Wörtern
		if r.randint(0,2):
			s = r.choice(geschmack) + r.choice(nahrung).lower()
		else:
			s = r.choice(voressen) + r.choice(nahrung).lower()
	else: # aus einem Wort bestehend
		if r.randint(0,2):
			s = r.choice(geschmack)
		else:
			s = r.choice(nahrung)
	return s


beilag = ['Soße', 'Sauce', 'Brühe', 'Brei', 'Püree', 'Kartoffeln', 'Salat', 'Marmelade']
beilag2 = ['Kroketten', 'Pommes', 'Rösti', 'Röstzwiebeln', 'Mayo', 'Mayonnaise', 'Ketchup', 'Bratkartoffeln', 'Zwiebelringen', 'Kartoffelpuffer', 'Soße', 'Nudeln', 'Gemüse', 'Knödeln', 'Salat', 'Marmelade']
def beilage():
	"""
	Gibt eine Beilage zum Essen zurück.

	.. only:: doctest

		>>> s = beilage()
		
		>>> assert re.match(re_wort, s)
	"""
	if r.randint(0,2):
		s = r.choice(beilag2)
	else:
		s = r.choice(geschmack) + r.choice(beilag).lower()
	return s


getraenk = ['Wasser', 'Saft', 'Tee', 'Kaffee', 'Eistee', 'Milch', 'Punsch', 'Bowle', 'Bier', 'Likör',  'Wein', 'Rum', 'Sekt', 'Schnaps', 'Cocktail']
def trinken():
	"""
	Gibt ein Getränk zurück.

	.. only:: doctest

		>>> s = trinken()
		
		>>> assert re.match(re_wort, s)
	"""
	if r.randint(0,2):
		s = r.choice(geschmack) + r.choice(getraenk).lower()
	else:
		s = r.choice(getraenk)
	return s


def stadt():
	"""
	Gibt eine Stadt zurück.

	.. only:: doctest

		>>> s = stadt()
		
		>>> assert re.match(r'^[A-Za-zäÄöÖüÜß/. -]+$', s)
	"""
	return r.choice(stadte).split(' (', 1)[0] # Stadt von Bundesland trennen


def stadt_bl():
	"""
	Gibt eine Stadt mit Bundesland zurück.

	.. only:: doctest

		>>> s = stadt_bl()

		testet ob ergebnis in liste ist
		>>> assert s in stadte
		
		>>> assert re.match(r'^[A-Za-zäÄöÖüÜß/.() -]+$', s)
	"""
	return r.choice(stadte)


gruppe = ['Menschen', 'Personen', 'Kinder', 'Tiere', 'Gedärme', 'Kadaver', 'Nudeln', 'Unterhosen', 'Würstchen', 'Bäume', 'Stühle', 'Schweine', 'Neger', 'Alkoholiker', 'Leichen']
def band():
	"""
	Gibt einen fiktiven Bandnamen zurück.

	.. only:: doctest

		>>> s = band()
		
		>>> assert re.match(r'^[A-Za-z0-9äÄöÖüÜß -]+$', s)
	"""
	z = r.randint(0,5)
	if z == 0:
		s = 'Die ' + str_add(adjektiv().capitalize(), 'en ') + r.choice(gruppe)
	if z == 1:
		s = r.choice(geschmack) + ' ' + ort()
	if z == 2:
		s = adjektiv().capitalize() + ' ' + ort()
	if z == 3:
		s = str_add(adjektiv().capitalize(), 'e ') + essen()
	if z == 4:
		s = adjektiv().capitalize()
	if z == 5:
		s = wort()
	return s


def bandart():
	"""
	Gibt eine Bandart zurück.

	Beispiel: 'Gothic Metal Band'

	.. only:: doctest

		>>> s = bandart()
		
		>>> assert re.match(r'^[A-Za-zäÄöÖüÜß -]+[Bb]and$', s)
	"""
	m = r.choice(musik)
	if ' ' in m:
		s = m + ' Band'
	elif '-' in m:
		s = m + '-Band'
	else:
		s = m + 'band'
	return s


def firma():
	"""
	Gibt einen fiktiven Firmenname zurück.

	.. todo::

		Funktion programmieren
	"""
	pass



#!/usr/bin/env python
# -*- coding: utf8 -*-

"""
pyzufall.person
---------------

Stellt die Klasse Person zur Verfügung. Mit ihr kann man ein Objekt erzeugen, dass eine Person mit zufällig generierten Daten darstellt.

Es kann auf jedes Attribut einzeln zugegriffen werden oder mit print(person) alle aufeinmal ausgeben werden.

Die generierten Daten basieren teilweise auf statistischen Werten und versuchen möglichst authentisch zu sein.

Quellen für Statistiken:

- https://www.destatis.de/
- http://de.statista.com/
- https://www.zensus2011.de/
- http://www.statistik2013.de/
"""

import random as r

from .helfer import chance, alter
from .generator import geschlecht, geburtsdatum, vorname_m, vorname_w, nachname, nickname, email, homepage, url, stadt, farbe, essen, beilage, sprichwort, beruf_m, beruf_w

# Daten
from .generator import interessen

class Person(object):
	"""
	Generiert Daten einer zufälligen und fiktiven Person.

	.. versionadded:: 0.9
	"""
	anzahl = 0
	debug = 0 # extern auf 1 setzen

	def __init__(self):
		self.geschlecht = geschlecht()
		if self.geschlecht:
			self.vorname = vorname_m() + chance(10, '-' + vorname_m()) # mit 10%iger Chance ein Doppelname
		else:
			self.vorname = vorname_w() + chance(10, '-' + vorname_w()) # mit 10%iger Chance ein Doppelname

		self.nachname = nachname()
		self.name = self.vorname + ' ' + self.nachname
		self.nickname = nickname(self.vorname, self.nachname)
		self.homepage = homepage(self.vorname, self.nachname, self.nickname)
		self.email = email(self.vorname, self.nachname, self.nickname, self.homepage)
		self.homepage = url(self.homepage) # http:// hinzufügen, nachdem die Domain für die E-Mail genutzt wurde
		self.geburtsdatum = geburtsdatum()
		self.geburtsort = stadt()
		self.alter = alter(self.geburtsdatum)
		if self.alter > 22 and r.randint(1, 100) < 60:
			self.geburtsname = nachname()
		else:
			self.geburtsname = self.nachname
		self.wohnort = stadt()
		self.beruf = Person._gen_beruf(self)
		self.interessen = Person._gen_interessen(self)
		self.lieblingsfarbe = farbe()
		self.lieblingsessen = essen() + chance(50, " mit " + beilage())
		self.motto = sprichwort()

		Person.anzahl += 1
		if Person.debug: print("Neue Person generiert: " + self.name)

	def __del__(self):
		Person.anzahl -= 1
		if Person.debug: print("Person '" + self.name + "' wurde gelöscht.")

	def __str__(self):
		s = "*" * 80 + "\n"
		s += "Name: " + self.name + " (" + self.nickname + ")\n"
		s += "Geburtsname: " + self.geburtsname
		s += "\nGeschlecht: "
		if self.geschlecht:
			s += "männlich"
		else:
			s += "weiblich"
		s += "\nGeburtsdatum: " + self.geburtsdatum + " (" + str(self.alter) + ")\n"
		s += "Geburtsort: " + self.geburtsort + "\n"
		s += "Wohnort: " + self.wohnort + "\n"
		s += "Beruf: " + self.beruf + "\n"
		s += "E-Mail: " + self.email + "\n"
		s += "Homepage: " + self.homepage + "\n"
		s += "Interessen: " + self.interessen + "\n"
		s += "Lieblingsfarbe: " + self.lieblingsfarbe + "\n"
		s += "Lieblingsessen: " + self.lieblingsessen + "\n"
		s += "Motto: " + self.motto + "\n"
		s += "*" * 80 + "\n"
		return s

	def _gen_beruf(self):
		"""
		Generiert den Beruf einer Person anhand des Alters und Statistiken.

		Es wird von 10% Arbeitslosigkeit ausgegangen, die ofizielle Statistik ist allerdings 7,10% im Jahr 2013.
		Quelle: http://de.statista.com/statistik/daten/studie/1224/umfrage/arbeitslosenquote-in-deutschland-seit-1995/

		Es wird angenommen, dass 30% der zwischen 19 und 29 jährigen studieren:
		Studienanfänger 43,3%
		Studienabsolventen: 26,2%
		Quelle: http://de.wikipedia.org/wiki/Abiturientenquote_und_Studienanf%C3%A4ngerquote
		"""
		if self.geschlecht: # männlich
			if self.alter < 6:
				return "kein"
			elif self.alter <= 18:
				return "Schüler"
			elif self.alter > 18 and self.alter < 30 and r.randint(0,100) <= 30: # 30% studieren
				return "Student"
			elif self.alter > 68:
				return "Rentner"
			elif r.randint(0,9): # 1 von 10 arbeitslos
				return beruf_m()
			else:
				return "arbeitslos"
		else: # weiblich
			if self.alter < 6:
				return "kein"
			elif self.alter <= 18:
				return "Schülerin"
			elif self.alter > 18 and self.alter < 30 and r.randint(0,100) <= 30: # 30% studieren
				return "Studentin"
			elif self.alter > 68:
				return "Rentnerin"
			elif r.randint(0,9): # 1 von 10 arbeitslos
				return beruf_w()
			else:
				return "arbeitslos"

	def _gen_interessen(self):
		"""
		Gibt eine Liste mit Interessen zurück.
		"""
		_anzahl = r.randint(1,3)
		if r.randint(0,1): # 50% haben mehr als 3 Interessen, maximal 8
			_anzahl + r.randint(1, 5)
		_s = ', '.join(r.sample(interessen, _anzahl))
		return _s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
pyzufall.helfer
---------------

Stellt diverse Hilfsfunktionen bereit.
"""

import os, sys
import random as r
from datetime import datetime, date

# Regex Pattern
re_wort = r'^[A-Za-zäÄöÖüÜß-]+$'
re_worte = r'^[A-Za-zäÄöÖüÜß -]+$'
re_liste = r'^[A-Za-zäÄöÖüÜß -]+,[A-Za-zäÄöÖüÜß, -]+'
re_datum = r'^(0[1-9]|[12][0-9]|3[01]).(0[1-9]|1[0-2]).(19|20)[0-9]{2}$'
re_email = r'^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$'
re_satz = r'^[A-ZÄÖÜ]{1}[a-zA-Z0-9éäÄöÖüÜß ,-/.?"()]+[.!?]{1}$'
re_frage = r'^[A-ZÄÖÜ]{1}[a-zA-Z0-9éäÄöÖüÜß ,-/"()]+[?]{1}$'


def lese(dateiname):
	"""
	Liest die Textdatei mit dem übergebenen Namen aus data/ zeilenweise ein und gib eine Liste zurück.

	Beispiel:

	>>> liste = lese('baeume.txt')

	`<http://stackoverflow.com/questions/10174211/make-an-always-relative-to-current-module-file-path>`_

	:param dateiname: Dateiname inklusive Endung, z.B. *vornamen.txt*
	:type dateiname: string
	:rtype: list

	.. only:: doctest

		# überprüfe ob liste eine liste ist
		>>> assert isinstance(liste, list)
	"""
	dateipfad = os.path.join(os.path.dirname(__file__), 'data/' + dateiname)
	return open(dateipfad, 'r').read().splitlines()


def chance(wahrscheinlichkeit, wert):
	"""
	Der übergebene Wert wird mit der gewählten Wahrscheinlichkeit zurückgegeben.

	.. versionadded:: 0.11
	
	:param wahrscheinlichkeit: int zwischen 1 und 100
	:param wert: string
	"""
	if r.randint(1, 100) <= wahrscheinlichkeit:
		return wert
	else:
		return ''


def erste_gross(s):
	"""
	Macht den ersten Buchstaben gross.

	Beispiele:

	>>> a = erste_gross('das ist ein Beispiel?')

	>>> print(a)
	Das ist ein Beispiel?

	>>> b = erste_gross('über Stock und Stein.')

	>>> print(b)
	Über Stock und Stein.

	>>> c = erste_gross('älter als das Internet!')

	>>> print(c)
	Älter als das Internet!
	"""
	if sys.version_info < (3, 0):
		# convert to unicode
		s = s.decode('utf-8')
		s = s[0].upper() + s[1:]
		# convert back
		return s.encode('utf-8')
	else:
		return s[0].upper() + s[1:]


def str_add(wort, string):
	"""
	Fügt einen String ans Ende eines Wortes an, ohne doppelte Buchstaben zu erzeugen.

	Beispiele:

	>>> a = str_add('feige', 'er')

	>>> print(a)
	feiger

	>>> b = str_add('feige', 'e')

	>>> print(b)
	feige

	>>> c = str_add('blöd', 'e')

	>>> print(c)
	blöde

	.. only:: docstest

		>>> str_add('unflexibel', 'e')
		'unflexible'
		>>> str_add('unflexibel', 'er')
		'unflexibler'

	.. versionadded:: 0.11
	"""
	# wenn der letzte Buchstabe des wortes ist gleich der erste des strings
	if wort[-1] == string[0]:
		# gebe wort + alles ohne den ersten des strings zurück
		s = wort + string[1:]
	else:
		s = wort + string

	# behebt Fehler: unflexibele -> unflexible
	if s[-3:] == 'ele':
		s = s[:-3] + 'le'

	# behebt Fehler: unflexibeler -> unflexibler
	if s[-4:] == 'eler':
		s = s[:-4] + 'ler'

	return s


def aufzaehlung(liste):
	"""
	Erzeugt eine grammatikalisch korrekte Aufzählung aus einer Liste.

	Beispiel:

	>>> a = ['lesen', 'reiten', 'Freunde treffen']

	>>> s = aufzaehlung(a)

	>>> print(s)
	lesen, reiten und Freunde treffen

	>>> b = ['Überwachen', 'Strafen']

	>>> s = aufzaehlung(b)

	>>> print(s)
	Überwachen und Strafen

	>>> c = ['schlafen']

	>>> s = aufzaehlung(c)

	>>> print(s)
	schlafen

	:param liste: Eine Liste von Strings.
	:type liste: list
	:rtype: string

	.. versionadded:: 0.12
	"""
	return "{}{}".format(', '.join(liste[:-2]) + ', ' if len(liste) > 2 else '', ' und '.join(liste[-2:]))


def alter(geburtsdatum):
	"""
	Berechnet das Alter in Jahren anhand des Geburtsdatums.

	:rtype: int

	.. only:: doctest

		>>> s = alter("27.10.1988")

		>>> assert s > 24

	.. versionadded:: 0.12
	"""
	_heute = date.today()
	_geburtstag = datetime.strptime(geburtsdatum, "%d.%m.%Y").date()
	_alter = int((_heute - _geburtstag).days / 365.2425)
	return _alter

def uml(s):
	"""
	Ersetzt Umlaute durch die entsprechenden 2 Buchstaben.

	Beispiel:

	>>> uml('Käse')
	'Kaese'
	>>> uml('Brötchen')
	'Broetchen'
	>>> uml('Gefühl')
	'Gefuehl'

	.. versionadded:: 0.13
	"""
	s = s.replace('ä', 'ae')
	s = s.replace('ü', 'ue')
	s = s.replace('ö', 'oe')
	s = s.replace('ß', 'ss')
	return s



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from pyzufall.version import __version__
from pyzufall import __doc__

setup(
    name='PyZufall',
    version=__version__,
    url='https://pyzufall.readthedocs.org',
    license='GPLv3',
    author='davidak',
    author_email='post@davidak.de',
    packages=['pyzufall'],
    package_dir={'pyzufall': 'pyzufall'},
    package_data={'pyzufall': ['data/*.txt']},
    platforms='any',
    description=__doc__,
    long_description=open('README.rst').read(),
    install_requires=[
    ],
    classifiers=[
        #'Development Status :: 1 - Planning',
        #'Development Status :: 2 - Pre-Alpha',
        #'Development Status :: 3 - Alpha',
        'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        #'Development Status :: 6 - Mature',
        #'Development Status :: 7 - Inactive',
        #'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        #'Programming Language :: Python :: 3.0',
        #'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        #'Programming Language :: Python :: Implementation :: IronPython',
        #'Programming Language :: Python :: Implementation :: Jython',
        'Programming Language :: Python :: Implementation :: PyPy',
        #'Programming Language :: Python :: Implementation :: Stackless',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: German',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Topic :: Artistic Software',
        'Topic :: Games/Entertainment',
        'Topic :: Text Processing :: Linguistic',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ],
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
pyzufall.satz
-------------

Stellt Funktionen bereit, die Sätze nach diversen Satz-Schemata generieren.

Die Funktion :py:func:`pyzufall.satz.satz()` beinhaltet alle.
"""

import re
import random as r

from .helfer import erste_gross, chance, str_add, aufzaehlung
from .generator import adjektiv, band, bandart, baum, beilage, beruf_m, beruf_w, beziehung_m, beziehung_w, color, datum, essen, farbe, firma, geburtsdatum, gegenstand, interesse, koerperteil, nachname, objekt, objekt_m, objekt_w, ort, person, person_m, person_objekt_m, person_objekt_w, person_w, pflanze, sprichwort, stadt, stadt_bl, tier, trinken, verbd, verbi, verbi2, verbn, verbt, verbt2, vorname, vorname_m, vorname_w, wort, zahl

# Daten
from .generator import adjektive, farben, interessen


def satz_nulltransitiv():
	"""
	Generiert einen Satz mit einem nulltransitiven Verb.

	Beispiel: Im Park schneit es.
	"""
	if r.randint(0,1):
		s = ort() + ' ' + verbn() + ' es' + chance(25, ' ' + adjektiv()) + '.'
	else:
		s = 'Es ' + verbn() + chance(25, ' ' + adjektiv()) + '.'
	return erste_gross(s)


def satz_baum():
	"""
	Generiert einen Satz mit dem Thema Baum.

	Beispiel: Die gnadenlose Kerstin tritt gegen den Apfelbaum.
	"""
	if r.randint(0,1): #männlich
		person = person_m()
		objekt = objekt_m(baum())
	else: # weiblich
		person = person_w()
		objekt = objekt_w(baum())
	s = person + ' ' + r.choice(['pflanzt', 'klettert auf', 'fällt', 'zersägt', 'stellt sich unter', 'umarmt', 'tritt gegen']) + ' ' + objekt + '.'
	return erste_gross(s)


besetzung = ['Sänger', 'Gitarrist', 'Keyboarder', 'Bassist', 'Schlagzeuger', 'Manager', 'Geiger', 'Trompeter', 'Saxophonist', 'Backgroundsänger']
def satz_band_mitglied():
	"""
	Generiert einen Satz, in dem ein Bandmitglied vorgestellt wird.

	Beispiel: Annelise ist Gitarristin von der Gothicband "Kräuter in der Innenstadt".
	"""
	if r.randint(0,1): # männlich
		s = vorname_m() + chance(50, ' ' + nachname()) + ' ist ' + chance(50, 'der ') + r.choice(besetzung) + chance(50, ' von') + ' der ' + bandart() + ' "' + band() + '".'
	else: # weiblich
		s = vorname_w() + chance(50, ' ' + nachname()) + ' ist ' + chance(50, 'die ') + r.choice(besetzung) + 'in' + chance(50, ' von') + ' der ' + bandart() + ' "' + band() + '".'
	return erste_gross(s)


def satz_band_gegruendet():
	"""
	Generiert einen Satz, der den Zeitpunkt einer Bandgründung zum Thema hat.

	Beispiel: Die Electroband "Kartoffel auf dem Klo" wurde am 26.10.2009 in Selb gegründet.
	"""
	if r.randint(0,1):
		s = 'Die ' + bandart() + ' "' + band() + '" wurde '
	else:
		s = band() + ' (' + bandart() + ') wurde '

	if r.randint(0,1): # 50% Datum oder Jahr
		s += 'am ' + datum()
	else:
		s += chance(75, r.choice(['Anfang', 'Mitte', 'Mitte des Jahres', 'Ende', 'im Frühling', 'im Sommer', 'im Herbst', 'im Winter', 'im Jahr'])) + ' ' + str(r.randint(1950, 2013))

	s += chance(50, ' in ' + stadt()) + ' gegründet.'
	return erste_gross(s)


def satz_band_besetzung():
	"""
	Generiert einen Satz mit den Mitgliedern einer Band.

	Beispiel: Die Black Metal Band "Die Oralen Nudeln" besteht aus Marlene, Gert, Stefanie, Timm, Andrej, Friederike und Dorothea.
	"""
	mit_nachname = r.randint(0,1)
	s = 'Die '+ bandart() + ' "' + band() + '" besteht aus ' + vorname()
	if mit_nachname:
		s += ' ' + nachname()
	for i in range(0,r.randint(0,6)): # 2 bis 8 Mitglieder
		s += ', ' + vorname()
		if mit_nachname:
			s += ' ' + nachname()
	s += ' und ' + vorname()
	if mit_nachname:
		s += ' ' + nachname()
	return erste_gross(s) + '.'


def satz_band():
	"""
	Generiert einen zufälligen Satz zum Thema Band.
	"""
	return r.choice([satz_band_mitglied, satz_band_gegruendet, satz_band_besetzung])()


def satz_arbeit():
	"""
	Generiert einen Satz über eine berufstätige Person.

	Beispiel: Achmed, der Grafiker aus Waldheim, spielt den Nasenbär.
	"""
	x = r.randint(1,4)
	if x == 1:
		s = vorname_m() + chance(50, ' ' + nachname()) + ' ist ' + chance(50, 'ein ') + beruf_m() + chance(50, ' aus ' + stadt())
	if x == 2:
		s = vorname_w() + chance(50, ' ' + nachname()) + ' ist ' + chance(75, 'eine ') + beruf_w() + chance(50, ' aus ' + stadt())
	if x == 3:
		s = vorname_m() + chance(50, ' ' + nachname()) + chance(50, ', der ' + beruf_m() + chance(50, ' aus ' + stadt()) + ',') + ' ' + verbt() + ' ' + r.choice([person_objekt_m(), objekt_m(objekt())])
	if x == 4:
		s = vorname_w() + chance(50, ' ' + nachname()) + chance(50, ', die ' + beruf_w() + chance(50, ' aus ' + stadt()) + ',') + ' ' + verbt() + ' ' + r.choice([person_objekt_w(), objekt_w(objekt())])
	return erste_gross(s) + '.'


def satz_essen():
	"""
	Generiert einen Satz mit Essen und/oder Trinken.

	Beispiel: Die Wärterin isst Orangen mit Mayonnaise und trinkt dazu Milch.
	"""
	x = r.randint(1,3)
	if x == 1: # Essen mit Beilage
		s = person() + chance(25, ' ' + r.choice(['ist', 'sitzt', 'befindet sich']) + ' ' + ort() + ' und') + ' isst ' + chance(50, r.choice(['gerade', 'oft', 'nicht oft', 'selten', 'gerne', 'nicht gerne']) + ' ') + essen() + chance(50, ' mit ' + r.choice(['viel', 'ganz viel', 'ein bischen', 'ein wenig', 'lecker', 'einer großen Portion']) + ' ' + beilage())
	if x == 2: # Trinken
		s = person() + ' trinkt ' + chance(50, r.choice(['gerade', 'oft', 'nicht oft', 'selten', 'gerne', 'nicht gerne', 'jeden Abend ein Glas', 'zu viel']) + ' ') + trinken()
	if x == 3: # Essen und Trinken
		s = person() + ' isst ' + essen() + chance(50, ' mit ' + beilage()) + ' und trinkt ' + chance(50, 'dazu ') + trinken()
	return erste_gross(s) + '.'


def satz_koerperteil():
	"""
	Generiert einen Satz zum Thema Körperteile.

	Beispiel: Die ekelhafte Oma massiert ihren Fuß.
	"""
	x = r.randint(1,5)
	if x == 1:
		if r.randint(0,1): # Er
			s = koerperteil()
			s = re.sub('der ', 'den ', s)
			if r.randint(0,1):
				s = re.sub('den ', 'seinen ', s)
				s = re.sub('die ', 'seine ', s)
				s = re.sub('das ', 'sein ', s)
			s = person_m() + ' ' + r.choice(['verletzt sich', 'stößt sich', 'bricht' + chance(50, ' sich'), 'verstaucht' + chance(50, ' sich'), 'massiert']) + ' ' + s
		else: # Sie
			s = koerperteil()
			s = re.sub('der ', 'den ', s)
			if r.randint(0,1):
				s = re.sub('den ', 'ihren ', s)
				s = re.sub('die ', 'ihre ', s)
				s = re.sub('das ', 'ihr ', s)
			s = person_w() + ' ' + r.choice(['verletzt sich', 'stößt sich', 'bricht' + chance(50, ' sich'), 'verstaucht' + chance(50, ' sich'), 'massiert']) + ' ' + s
	if x == 2: # hat Kopfschmerzen
		s = koerperteil()
		s = re.sub('e$', 'en', s) # Lungen
		s = s.split()[1] # Körperteil ohne Artikel
		s = person() + ' hat ' + chance(50, r.choice(['schlimme', 'ein bischen', 'brutale', 'oft', 'manchmal']) + ' ') + s + 'schmerzen'
	if x == 3:
		s = koerperteil()
		s = re.sub('der ', 'am ', s)
		s = re.sub('die ', 'an der ', s)
		s = re.sub('das ', 'am ', s)
		s = person() + ' hat eine ' + chance(25, r.choice(['schmerzhafte ', 'brutale ', 'miese ', 'kleine ', 'eiternde '])) + r.choice(['Verletzung', 'Entzündung', 'Wunde', 'Beschädigung']) + ' ' + s
	if x == 4:
		s = koerperteil()
		s = re.sub('der ', 'Mein ', s)
		s = re.sub('die ', 'Meine ', s)
		s = re.sub('das ', 'Mein ', s)
		s = s + ' ' + r.choice(['tut weh', 'schmerzt', 'blutet', 'juckt', 'brennt', 'stinkt'])
	if x == 5:
		s = koerperteil()
		s = re.sub('der ', 'einen ' + str_add(adjektiv(), 'en '), s)
		s = re.sub('die ', 'eine ' + str_add(adjektiv(), 'e '), s)
		s = re.sub('das ', 'ein ' + str_add(adjektiv(), 'es '), s)
		s = person() + ' hat ' + s
	return erste_gross(s) + '.'


def satz_kloster():
	"""
	Generiert einen Satz über eine Person in einem Kloster.

	Beispiel: Bruder Ludwig ist der böseste Mönch im Kloster.
	"""
	_adj = adjektiv()
	# verhindert 'monströsste' und 'gerechtste'
	if _adj[-1] == 's' or _adj[-1] == 't':
		_adj += 'este'
	else:
		_adj += 'ste'
	if r.randint(0,1): # männlich
		s = chance(25, 'Der ') + 'Bruder ' + vorname_m() + r.choice([' war', ' ist']) + ' der ' + _adj + ' Mönch ' + r.choice(['im Kloster', 'im Orden', 'in der Abtei'])
	else: # weiblich
		s = chance(25, 'Die ') + 'Schwester ' + vorname_w() + r.choice([' war', ' ist']) + ' die ' + _adj + ' Nonne ' + r.choice(['im Kloster', 'im Orden', 'in der Abtei'])
	return erste_gross(s) + '.'


def satz_folgehandlung():
	"""
	Generiert einen Satz, der eine Folgehandlung beschreibt.

	Beispiel: Ohne dass Irmgard überlebt, bricht sie aggressiv ein.
	"""
	v1, v2 = verbi2().split(", ")
	v12 = ''
	if ' ' in v1: # bildet sich => bilder ER sich
		v1, v12 = v1.split(' ')
		v12 += ' '
	
	verb2 = verbi()
	verb22 = ' '
	if ' ' in verb2:
		verb2, verb22 = verb2.split(' ')
		verb22 = ' ' + verb22 + ' '
	
	if r.randint(0,3): # Selbst
		if r.randint(0,1): # m
			erste_person, zweit_person = person_m(), ' er '
		else: # w
			erste_person, zweit_person = person_w(), ' sie '
	else: # Dritte
		if r.randint(0,1): # m
			erste_person, zweit_person = person_m(), r.choice([' sein ' + r.choice(beziehung_m), ' seine ' + r.choice(beziehung_w)]) + ' '
		else: # w
			erste_person, zweit_person = person_w(), r.choice([' ihr ' + r.choice(beziehung_m), ' ihre ' + r.choice(beziehung_w)]) + ' '
	
	s = r.choice(['Weil', 'Während', 'Obwohl', 'Ohne dass', 'Nur weil', 'Gerade weil']) + r.choice([verb22 + erste_person + ' ', ' ' + erste_person + verb22]) + chance(50, ort() + ' ') + chance(50, adjektiv() + ' ') + verb2 + ', ' + v1 + zweit_person + v12 + chance(50, adjektiv() + ' ') + v2 + '.'
	return erste_gross(s)


def satz_absurde_farbfunktion():
	"""
	Generiert einen Satz nach folgendem Muster: Gelb ist brauner als Türkis.
	"""
	farbe1, farbe2, farbe3 = r.sample(farben, 3)
	s = farbe1 + ' ist ' + str_add(r.choice([farbe2.lower, adjektiv])(), 'er') + ' als ' + farbe3 + '.'
	return erste_gross(s)


def satz_freunde_lieben():
	"""
	Generiert einen Satz über eine Person mit Eigenschaften.

	Beispiel: In der Garage ist das Mannsweib lesbisch.
	"""
	if r.randint(0,1): #männlich
		s = person_m() + ' ist ' + adjektiv() + chance(50, r.choice(['. Alle', ', aber']) + ' seine Freunde lieben ihn dafür') + '.'
	else: #weiblich
		s = person_w() + ' ist ' + adjektiv() + chance(50, r.choice(['. Alle', ', aber']) + ' ihre Freunde lieben sie dafür') + '.'
	return erste_gross(s)


def satz_farbe():
	"""
	Generiert einen Satz nach dem Muster: Braun ist eine unsittliche Farbe.
	"""
	s = farbe() + ' ist eine ' + str_add(adjektiv(), 'e ') + 'Farbe.'
	return erste_gross(s)


def satz_adjektiv_sprichwort():
	"""
	Generiert einen Satz nach dem Muster: Je untrainierter desto lächerlicher.
	"""
	_adj1, _adj2 = r.sample(adjektive, 2)
	_s = 'Je ' + str_add(_adj1, 'er') + ' desto ' + str_add(_adj2, 'er.')
	return erste_gross(_s)


def satz_adjektiv_am_ort():
	"""
	Generiert einen Satz nach dem Muster: <Ort> <Verb> <Person> <Adjektiv>.

	Beispiel: Auf dem Spielplatz ist die Freundin hilfsbereit.
	"""
	s = ort() + ' ' + r.choice(['war', 'ist']) + ' ' + person() + ' ' + chance(50, r.choice(['sehr', 'ziemlich', 'ein bischen', 'nicht sehr', 'garnicht', 'nicht']) + ' ') + adjektiv() + '.'
	return erste_gross(s)


def satz_interessen():
	"""
	Generiert einen Satz über die Interessen einer Person.
	"""
	if r.randint(0,1):
		_s = "{} Interessen sind {}.".format(str_add(vorname(), 's'), aufzaehlung(r.sample(interessen, r.randint(2,6))))
	else:
		_s = "{} zählt zu {} Interessen.".format(interesse(), str_add(vorname(), 's'))
	return erste_gross(_s)


def satz_thema():
	"""
	Generiert einen Satz zu einem zufälligen Thema.
	"""
	return r.choice([satz_nulltransitiv, satz_baum, satz_band, satz_kloster, satz_folgehandlung, satz_freunde_lieben, satz_adjektiv_am_ort, satz_absurde_farbfunktion, satz_farbe, satz_adjektiv_sprichwort, satz_arbeit, satz_essen, satz_koerperteil, satz_interessen])()


def satz_standard_1():
	"""
	Generiert einen einfachen Satz nach dem Muster: <Person> <Verb> <Adjektiv> <Ort>.

	Beispiel: Die Geschmacklose bepisst sich cool in der Kirche.
	"""
	if r.randint(0,1):
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = person() + ' ' + v1 + ' ' + r.choice([adjektiv(), ort(), adjektiv() + ' ' + ort()]) + v2 + '.'
	return erste_gross(s)


def satz_standard_2():
	"""
	Generiert einen einfachen Satz nach dem Muster: <Ort> <Verb> <Person> <Adjektiv>.

	Beispiel: Beim ersten Date flieht er.
	"""
	if r.randint(0,1):
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = ort() + ' ' + v1 + ' ' + person() + chance(75, ' ' + adjektiv()) + v2 + '.'
	return erste_gross(s)


def satz_standard_3():
	"""
	Generiert einen einfachen Satz nach dem Muster: <Adjektiv> <Verb> <Person> <Ort>.

	Beispiel: Gehirntot weint die Schädlingsbekämpferin in der Psychiatrie.
	"""
	if r.randint(0,1):
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = adjektiv() + ' ' + v1 + ' ' + person() + chance(75, ' ' + ort()) + v2 + '.'
	return erste_gross(s)


def satz_standard_4():
	"""
	Generiert einen einfachen Satz nach dem Muster: <Person> <Verb> <Person/Objekt> <Adjektiv> <Ort>.

	Beispiel: Der Ruhige raubt ein Schaf aus.
	"""
	if r.randint(0,1): # männlich
		_person = person_m()
		_objekt = r.choice([person_objekt_m(), objekt_m(objekt())])
	else: # weiblich
		_person = person_w()
		_objekt = r.choice([person_objekt_w(), objekt_w(objekt())])

	if r.randint(0,1): # getrenntes Verb
		v1, v2 = verbt2().split(",")
	else:
		v1, v2 = verbt(), ''

	s = _person + ' ' + v1 + ' ' + _objekt + r.choice([' ' + adjektiv(), ' ' + ort(), ' ' + adjektiv() + ' ' + ort(), '']) + v2 + '.'
	return erste_gross(s)
	

def satz_standard():
	"""
	Generiert einen zufälligen Standard-Satz.
	"""
	_funktion = 'satz_standard_' + str(r.randint(1, 4))
	return eval(_funktion)()


def satz_frage_1():
	"""
	Generiert eine Frage nach dem Grund, aus dem eine Person eine Tätigkeit ausführt

	Beispiel: Wieso fällt dein Partner in Gedanken hin?
	"""
	if r.randint(0,1): # getrenntes Verb
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = chance(50, r.choice(['Warum ', 'Wieso ', 'Weshalb '])) + v1 + ' ' + person() + r.choice([' ' + adjektiv(), ' ' + ort(), ' ' + adjektiv() + ' ' + ort(), '']) + v2 + '?'
	return erste_gross(s)


def satz_frage_2():
	"""
	Generiert eine Frage nach der Person, die eine Tätigkeit ausführt.

	Beispiel: Wer telefoniert bewusstlos in der Abtei?
	"""
	if r.randint(0,1): # getrenntes Verb
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = 'Wer ' + v1 + ' ' + r.choice([adjektiv(), ort(), adjektiv() + ' ' + ort()]) + v2 + '?'
	return erste_gross(s)


def satz_frage_3():
	"""
	Generiert eine Frage nach dem Ort, an dem eine Person eine Tätigkeit ausführt.

	Beispiel: Wo singt ein Siebdrucker?
	"""
	if r.randint(0,1): # getrenntes Verb
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = 'Wo ' + v1 + ' ' + person() + chance(25, ' ' + adjektiv()) + v2 + '?'
	return erste_gross(s)


def satz_frage_4():
	"""
	Generiert eine Frage nach der Art, wie eine Person eine Tätigkeit ausführt.

	Beispiel: Wie wird sie beim ersten Date angefasst?
	"""
	if r.randint(0,1): # getrenntes Verb
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = 'Wie ' + v1 + ' ' + person() + chance(50, ' ' + ort()) + v2 + '?'
	return erste_gross(s)


def satz_frage_5():
	"""
	Generiert eine Frage nach dem Zeitpunkt, an dem eine Person eine Tätigkeit ausführt.

	Beispiel: Wann säuft eine Hure?
	"""
	if r.randint(0,1):
		v1, v2 = verbi2().split(",")
	else:
		v1, v2 = verbi(), ''
	s = 'Wann ' + v1 + ' ' + person() + chance(50, ' endlich') + chance(25, ' ' + ort()) + v2 + '?'
	return erste_gross(s)


def satz_frage():
	"""
	Generiert eine zufällige Frage.
	"""
	_funktion = 'satz_frage_' + str(r.randint(1, 5))
	return eval(_funktion)()


def satz():
	"""
	Generiert einen zufälligen Satz.

	20% Standard-Sätze, 20% Fragen und 60% Themen-Sätze
	"""
	x = r.randint(1,5)

	if x == 1:
		s = satz_standard()
	elif x == 2:
		s = satz_frage()
	else:
		s = satz_thema()

	return s
